'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadCodeVersion = exports.ExTester = exports.NODEJS_VERSION_MAX = exports.VSCODE_VERSION_MAX = exports.VSCODE_VERSION_MIN = exports.DEFAULT_STORAGE_FOLDER = exports.DEFAULT_SETUP_OPTIONS = exports.ReleaseQuality = void 0;
const codeUtil_1 = require("./util/codeUtil");
Object.defineProperty(exports, "ReleaseQuality", { enumerable: true, get: function () { return codeUtil_1.ReleaseQuality; } });
const driverUtil_1 = require("./util/driverUtil");
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const os = __importStar(require("os"));
const url_1 = require("url");
__exportStar(require("./browser"), exports);
__exportStar(require("./suite/mochaHooks"), exports);
__exportStar(require("monaco-page-objects"), exports);
exports.DEFAULT_SETUP_OPTIONS = {
    vscodeVersion: 'latest',
    installDependencies: false
};
exports.DEFAULT_STORAGE_FOLDER = process.env.TEST_RESOURCES ? process.env.TEST_RESOURCES : path.join(os.tmpdir(), 'test-resources');
exports.VSCODE_VERSION_MIN = '1.84.2';
exports.VSCODE_VERSION_MAX = '1.86.1';
/**
 * The latest version of NodeJS which is properly working with selenium-webdriver
 * (for more details, see https://www.npmjs.com/package/selenium-webdriver?activeTab=readme#node-support-policy)
 */
exports.NODEJS_VERSION_MAX = '18.15.0';
/**
 * ExTester
 */
class ExTester {
    constructor(storageFolder = exports.DEFAULT_STORAGE_FOLDER, releaseType = codeUtil_1.ReleaseQuality.Stable, extensionsDir) {
        this.code = new codeUtil_1.CodeUtil(storageFolder, releaseType, extensionsDir);
        this.chrome = new driverUtil_1.DriverUtil(storageFolder);
        if (process.versions.node > exports.NODEJS_VERSION_MAX) {
            console.log('\x1b[31m%s\x1b[0m', `\nERROR:\tYou are using the unsupported NodeJS version '${process.versions.node}'. The latest supported version is '${exports.NODEJS_VERSION_MAX}'.\n\tWe recommend to use supported version to have ExTester working properly.\n\tMore info at https://github.com/redhat-developer/vscode-extension-tester/issues/975\n\n`);
        }
    }
    /**
     * Download VSCode of given version and release quality stream
     * @param version version to download, default latest
     */
    downloadCode(version = 'latest') {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.code.downloadVSCode(loadCodeVersion(version));
        });
    }
    /**
     * Install the extension into the test instance of VS Code
     * @param vsixFile path to extension .vsix file. If not set, default vsce path will be used
     * @param useYarn when true run `vsce package` with the `--yarn` flag
     */
    installVsix({ vsixFile, useYarn, installDependencies } = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let target = vsixFile;
            if (vsixFile) {
                try {
                    const uri = new url_1.URL(vsixFile);
                    if (!(process.platform === 'win32' && /^\w:/.test(uri.protocol))) {
                        target = path.basename(vsixFile);
                    }
                }
                catch (err) {
                    if (!fs.existsSync(vsixFile)) {
                        throw new Error(`File ${vsixFile} does not exist`);
                    }
                }
                if (target !== vsixFile) {
                    target = yield this.code.downloadExtension(vsixFile);
                }
            }
            else {
                yield this.code.packageExtension(useYarn);
            }
            this.code.installExtension(target);
            if (installDependencies) {
                this.code.installDependencies();
            }
        });
    }
    /**
     * Install an extension from VS Code marketplace into the test instance
     * @param id id of the extension to install
     */
    installFromMarketplace(id) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.code.installExtension(undefined, id);
        });
    }
    /**
     * Download the matching chromedriver for a given VS Code version
     * @param vscodeVersion selected versio nof VSCode, default latest
     */
    downloadChromeDriver(vscodeVersion = 'latest') {
        return __awaiter(this, void 0, void 0, function* () {
            const chromiumVersion = yield this.code.getChromiumVersion(loadCodeVersion(vscodeVersion));
            return yield this.chrome.downloadChromeDriverForChromiumVersion(chromiumVersion);
        });
    }
    /**
     * Performs all necessary setup: getting VSCode + ChromeDriver
     * and packaging/installing extension into the test instance
     *
     * @param options Additional options for setting up the tests
     */
    setupRequirements(options = exports.DEFAULT_SETUP_OPTIONS, offline = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const { useYarn, vscodeVersion, installDependencies } = options;
            const vscodeParsedVersion = loadCodeVersion(vscodeVersion);
            if (!offline) {
                yield this.downloadCode(vscodeParsedVersion);
                yield this.downloadChromeDriver(vscodeParsedVersion);
            }
            else {
                console.log('Attempting Setup in offline mode');
                const expectedChromeVersion = (yield this.code.checkOfflineRequirements()).split('.')[0];
                const actualChromeVersion = (yield this.chrome.checkDriverVersionOffline(vscodeParsedVersion)).split('.')[0];
                if (expectedChromeVersion !== actualChromeVersion) {
                    console.log('\x1b[33m%s\x1b[0m', `WARNING: Local copy of VS Code runs Chromium version ${expectedChromeVersion}, the installed ChromeDriver is version ${actualChromeVersion}.`);
                    console.log(`Attempting with ChromeDriver ${actualChromeVersion} anyway. Tests may experience issues due to version mismatch.`);
                }
            }
            yield this.installVsix({ useYarn });
            if (installDependencies && !offline) {
                this.code.installDependencies();
            }
        });
    }
    /**
     * Performs requirements setup and runs extension tests
     *
     * @param testFilesPattern glob pattern(s) for test files to run
     * @param vscodeVersion version of VS Code to test against, defaults to latest
     * @param setupOptions Additional options for setting up the tests
     * @param runOptions Additional options for running the tests
     *
     * @returns Promise resolving to the mocha process exit code - 0 for no failures, 1 otherwise
     */
    setupAndRunTests(testFilesPattern, vscodeVersion = 'latest', setupOptions = exports.DEFAULT_SETUP_OPTIONS, runOptions = codeUtil_1.DEFAULT_RUN_OPTIONS) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setupRequirements(Object.assign(Object.assign({}, setupOptions), { vscodeVersion }), runOptions.offline);
            return yield this.runTests(testFilesPattern, Object.assign(Object.assign({}, runOptions), { vscodeVersion }));
        });
    }
    /**
     * Runs the selected test files in VS Code using mocha and webdriver
     * @param testFilesPattern glob pattern(s) for selected test files
     * @param runOptions Additional options for running the tests
     *
     * @returns Promise resolving to the mocha process exit code - 0 for no failures, 1 otherwise
     */
    runTests(testFilesPattern, runOptions = codeUtil_1.DEFAULT_RUN_OPTIONS) {
        return __awaiter(this, void 0, void 0, function* () {
            runOptions.vscodeVersion = loadCodeVersion(runOptions.vscodeVersion);
            const patterns = (typeof testFilesPattern === 'string') ? ([testFilesPattern]) : (testFilesPattern);
            return yield this.code.runTests(patterns, runOptions);
        });
    }
}
exports.ExTester = ExTester;
function loadCodeVersion(version) {
    const code_version = process.env.CODE_VERSION ? process.env.CODE_VERSION : version;
    if (code_version !== undefined) {
        if (code_version.toLowerCase() === 'max') {
            return exports.VSCODE_VERSION_MAX;
        }
        if (code_version.toLowerCase() === 'min') {
            return exports.VSCODE_VERSION_MIN;
        }
        return code_version;
    }
    return 'latest';
}
exports.loadCodeVersion = loadCodeVersion;
//# sourceMappingURL=extester.js.map