'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DriverUtil = void 0;
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const child_process = __importStar(require("child_process"));
const unpack_1 = require("./unpack");
const download_1 = require("./download");
const extester_1 = require("../extester");
/**
 * Handles version checks and download of ChromeDriver
 */
class DriverUtil {
    /**
     * Create an instance of chrome driver handler
     * @param folder path to a folder to store all artifacts
     */
    constructor(folder = extester_1.DEFAULT_STORAGE_FOLDER) {
        // older chromedriver versions do not match chrome versions
        this.chromiumVersionMap = {
            69: '2.38',
            68: '2.38',
            67: '2.38',
            66: '2.38',
            65: '2.37',
            64: '2.36',
            63: '2.35',
            62: '2.34',
            61: '2.33',
            60: '2.32'
        };
        this.downloadFolder = path.resolve(folder);
    }
    /**
     * Find a matching ChromeDriver version for a given Chromium version and download it.
     * @param chromiumVersion version of Chromium to match the ChromeDriver against
     */
    downloadChromeDriverForChromiumVersion(chromiumVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            const version = yield this.getChromeDriverVersion(chromiumVersion);
            return yield this.downloadChromeDriver(version);
        });
    }
    /**
     * Download a given version ChromeDriver
     * @param version version to download
     */
    downloadChromeDriver(version) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = this.getChromeDriverURL(version);
            const driverBinary = this.getChromeDriverBinaryPath(version);
            if (fs.existsSync(driverBinary)) {
                let localVersion = '';
                try {
                    localVersion = yield this.getLocalDriverVersion(version);
                }
                catch (err) {
                    // ignore and download
                }
                if (localVersion.startsWith(version)) {
                    console.log(`ChromeDriver ${version} exists in local cache, skipping download`);
                    return '';
                }
            }
            fs.mkdirpSync(this.downloadFolder);
            const fileName = path.join(this.downloadFolder, path.basename(url));
            console.log(`Downloading ChromeDriver ${version} from: ${url}`);
            yield download_1.Download.getFile(url, fileName, true);
            console.log(`Unpacking ChromeDriver ${version} into ${this.downloadFolder}`);
            yield unpack_1.Unpack.unpack(fileName, this.downloadFolder);
            if (process.platform !== 'win32') {
                fs.chmodSync(driverBinary, 0o755);
            }
            console.log('Success!');
            return driverBinary;
        });
    }
    getChromeDriverBinaryPath(version) {
        const majorVersion = this.getMajorVersion(version);
        const binary = process.platform === 'win32' ? 'chromedriver.exe' : 'chromedriver';
        let driverBinaryPath = path.join(this.downloadFolder, binary);
        if (+majorVersion > 114) {
            driverBinaryPath = path.join(this.downloadFolder, `chromedriver-${DriverUtil.getChromeDriverPlatform()}`, binary);
        }
        return driverBinaryPath;
    }
    static getChromeDriverPlatform() {
        switch (process.platform) {
            case 'darwin':
                return `mac-${process.arch}`;
            case 'win32':
                return process.arch === 'x64' ? 'win64' : 'win32';
            case 'linux':
                return 'linux64';
            default:
                break;
        }
        return undefined;
    }
    static getChromeDriverPlatformOLD() {
        switch (process.platform) {
            case 'darwin':
                return process.arch === 'arm64' ? 'mac_arm64' : 'mac64';
            case 'win32':
                return 'win32';
            case 'linux':
                return 'linux64';
            default:
                break;
        }
        return undefined;
    }
    getChromeDriverURL(version) {
        const majorVersion = this.getMajorVersion(version);
        let driverPlatform = DriverUtil.getChromeDriverPlatformOLD();
        let url = `https://chromedriver.storage.googleapis.com/${version}/chromedriver_${driverPlatform}.zip`;
        if (+majorVersion > 114) {
            driverPlatform = DriverUtil.getChromeDriverPlatform();
            url = `https://edgedl.me.gvt1.com/edgedl/chrome/chrome-for-testing/${version}/${driverPlatform}/chromedriver-${driverPlatform}.zip`;
        }
        return url;
    }
    checkDriverVersionOffline(version) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.getLocalDriverVersion(version);
            }
            catch (err) {
                console.log('ERROR: Cannot find a copy of ChromeDriver in local cache in offline mode, exiting.');
                throw err;
            }
        });
    }
    /**
     * Check local chrome driver version
     */
    getLocalDriverVersion(version) {
        return __awaiter(this, void 0, void 0, function* () {
            const command = `${this.getChromeDriverBinaryPath(version)} -v`;
            return new Promise((resolve, reject) => {
                child_process.exec(command, (err, stdout) => {
                    if (err)
                        return reject(err);
                    resolve(stdout.split(' ')[1]);
                });
            });
        });
    }
    /**
     * Find a matching version of ChromeDriver for a given Chromium version
     * @param chromiumVersion Chromium version to check against
     */
    getChromeDriverVersion(chromiumVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            const majorVersion = this.getMajorVersion(chromiumVersion);
            // chrome driver versioning has changed for chrome 70+
            if (+majorVersion < 70) {
                if (this.chromiumVersionMap[+majorVersion]) {
                    return this.chromiumVersionMap[+majorVersion];
                }
                else {
                    throw new Error(`Chromium version ${chromiumVersion} not supported`);
                }
            }
            let url = `https://chromedriver.storage.googleapis.com/LATEST_RELEASE_${majorVersion}`;
            if (+majorVersion > 114) {
                url = `https://googlechromelabs.github.io/chrome-for-testing/LATEST_RELEASE_${majorVersion}`;
            }
            const fileName = 'driverVersion';
            yield download_1.Download.getFile(url, path.join(this.downloadFolder, fileName));
            return fs.readFileSync(path.join(this.downloadFolder, fileName)).toString();
        });
    }
    getMajorVersion(version) {
        return version.split('.')[0];
    }
}
exports.DriverUtil = DriverUtil;
//# sourceMappingURL=driverUtil.js.map